let ActionTypes = {};
ActionTypes.empty = 0;
ActionTypes.start = 1;
ActionTypes.cancel = 2;
ActionTypes.success = 3;
ActionTypes.failure = 4;
ActionTypes[ActionTypes.empty] = "empty";
ActionTypes[ActionTypes.start] = "start";
ActionTypes[ActionTypes.cancel] = "cancel";
ActionTypes[ActionTypes.success] = "success";
ActionTypes[ActionTypes.failure] = "failure";

/**
 * @license Redux-Typed-Actions
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
const { keys } = Object;
class ClassActionFactory {
    constructor() {
        this.suffixes = {};
        this.registry = {};
        /* tslint:disable:strict-type-predicates */
        if (typeof process !== 'undefined' && process.env != undefined && process.env.NODE_ENV === 'production') {
            this.env = 'production';
        }
        else {
            this.env = 'development';
        }
        /* tslint:enable:strict-type-predicates */
        this.setSuffixes({
            empty: '',
            start: '',
            cancel: ' Cancel',
            success: ' Success',
            failure: ' Failure',
        });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setEnvironment(value) {
        this.env = value;
    }
    /**
     * @param {?} inputs
     * @return {?}
     */
    setSuffixes(inputs) {
        for (const /** @type {?} */ key of keys(inputs)) {
            this.suffixes[key] = inputs[key];
        }
    }
    /**
     * @param {?} input
     * @return {?}
     */
    getSuffix(input) {
        return this.suffixes[ActionTypes[input]];
    }
    /**
     * @param {?} name
     * @param {?} type
     * @return {?}
     */
    registerTypeName(name, type) {
        const /** @type {?} */ actionTypeName = this.getTypeName(name, type);
        if (this.registry[actionTypeName]) {
            throw new Error(`Action with type name as "${name}" has already registered. You need define unique actions!`); // `;
        }
        else {
            this.registry[actionTypeName] = true;
        }
    }
    /**
     * @param {?} name
     * @param {?} input
     * @return {?}
     */
    getTypeName(name, input) {
        const /** @type {?} */ actionTypeName = name + this.getSuffix(input);
        return actionTypeName;
    }
}
const factory = new ClassActionFactory();

/**
 * @license Redux-Typed-Actions
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
const { defineProperties: defineProperties$1 } = Object;
/**
 * @param {?} input
 * @return {?}
 */
function getSafeClassName(input) {
    let /** @type {?} */ inputAsString = input.toString();
    const /** @type {?} */ match = /^symbol\((.*)\)$/i.exec(inputAsString);
    if (match !== null) {
        inputAsString = `SymbolAction_${match[1]}`; // `;
    }
    return inputAsString.replace(/\W/g, '_');
}
/**
 * @param {?} input
 * @return {?}
 */
function generateClassAction(input) {
    const /** @type {?} */ actionClassName = getSafeClassName(input);
    const /** @type {?} */ actionClass = (new Function(`return function ${actionClassName}() { }`))(); // `;
    return actionClass;
}
/**
 * @param {?} classAction
 * @param {?} actionTypeName
 * @param {?} getMethod
 * @return {?}
 */
function defineClassActionProperties(classAction, actionTypeName, getMethod) {
    const /** @type {?} */ isMethod = (action) => {
        return action.type === actionTypeName;
    };
    const /** @type {?} */ castMethod = (action) => {
        return action;
    };
    defineProperties$1(classAction, {
        /**
         * Defines a static property to get the "type" key
         * of this action
         */
        type: {
            writable: false,
            enumerable: true,
            configurable: false,
            value: actionTypeName,
        },
        /**
         * Defines a static method to check if a plain action
         * has the same type key as this defined action.
         */
        is: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: isMethod,
        },
        /**
         * Returns the plain action passed as argument with proper typings
         */
        cast: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: castMethod,
        },
        /**
         * Returns a plain action with a proper typed payload.
         */
        get: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: getMethod,
        },
        /**
         * Returns a plain action with a proper typed payload.
         */
        strictGet: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: getMethod,
        },
    });
}

/**
 * @license Redux-Typed-Actions v0.1.4
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
const { defineProperties } = Object;
/**
 *
 * @template Payload
 * @param {?} actionTypeName
 * @return {?}
 */
function defineAction(actionTypeName) {
    const /** @type {?} */ prefixedActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.empty);
    const /** @type {?} */ classAction = generateClassAction(prefixedActionTypeName);
    const /** @type {?} */ getMethod = (payload = undefined, meta = undefined, error = false) => {
        return {
            type: actionTypeName,
            payload,
            meta,
            error,
        };
    };
    defineClassActionProperties(classAction, actionTypeName, getMethod);
    return /** @type {?} */ (classAction);
}
/**
 *
 * @template Payload, SuccessPayload, FailurePayload, CancelPayload
 * @param {?} actionTypeName
 * @return {?}
 */
function defineScenarioAction(actionTypeName) {
    const /** @type {?} */ startActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.start);
    const /** @type {?} */ cancelActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.cancel);
    const /** @type {?} */ successActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.success);
    const /** @type {?} */ failureActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.failure);
    const /** @type {?} */ startClassAction = generateClassAction(startActionTypeName);
    const /** @type {?} */ cancelClassAction = generateClassAction(cancelActionTypeName);
    const /** @type {?} */ successClassAction = generateClassAction(successActionTypeName);
    const /** @type {?} */ failureClassAction = generateClassAction(failureActionTypeName);
    const /** @type {?} */ getMethod = (type = '', error = false, payload = undefined, meta = undefined) => {
        return {
            type,
            payload,
            meta,
            error,
        };
    };
    defineClassActionProperties(startClassAction, startActionTypeName, getMethod.bind(null, startActionTypeName, false));
    defineClassActionProperties(cancelClassAction, cancelActionTypeName, getMethod.bind(null, cancelActionTypeName, false));
    defineClassActionProperties(successClassAction, successActionTypeName, getMethod.bind(null, successActionTypeName, false));
    defineClassActionProperties(failureClassAction, failureActionTypeName, getMethod.bind(null, failureActionTypeName, true));
    defineProperties(startClassAction, {
        /**
         * Returns a simple class action for cancel
         */
        cancel: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: cancelClassAction,
        },
        /**
         * Returns a simple class action for success
         */
        success: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: successClassAction,
        },
        /**
         * Returns a simple class action for failure
         */
        failure: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: failureClassAction,
        },
    });
    return /** @type {?} */ (startClassAction);
}
/**
 *
 * @template P
 * @param {?} actionTypeName
 * @return {?}
 */
function defineSymbolAction(actionTypeName) {
    return /** @type {?} */ (defineAction(/** @type {?} */ (Symbol(actionTypeName))));
}

/**
 * Generated bundle index. Do not edit.
 */

export { factory, ClassActionFactory as __ClassActionFactory, ActionTypes, defineAction, defineScenarioAction, defineSymbolAction };
//# sourceMappingURL=redux-typed-actions.js.map
